/*////////////////////////////////////////////////////////////////////////////
    PROJECT:    Energy Monitoring System
    COPYRIGHT    2011, 2012 Dean J. Boman

    FILE:       Main.c
    AUTHOR:     Dean J. Boman

    OVERVIEW:
        Implementation of the main routine for the Energy Monitoring system.  This routine calls
		and configures various functions from the Microchip TCP/IP stack version 5.36.2.
		In addition it implements application specific functions for the Energy Monitoring System.

		This routine is based on the Microchip maindemo.c function will all the unused
		defines, ifdefs, and function calls removed for a cleaner implementation.



    REVISION HISTORY:

	Oct. 12. 2011	Initial development based on stack version 5.36.2
	Dec. 10, 2011	Ported stack from System Monitor to Energy Monitoring System.  Updated for PIC32.
	March 10, 2012	Updated with EMS functionality
	March 18, 2012	Updated based on hardware/software integration
	April 26, 2012	Add TCP command interface for WebServer



*/////////////////////////////////////////////////////////////////////////////

#define THIS_IS_STACK_APPLICATION

// Include all headers for any enabled TCPIP Stack functions
#include "TCPIP Stack/TCPIP.h"


// Include functions specific to this stack application
#include "Main.h"
#include "typedef.h"
#include "Time_conversion.h"
#include <math.h>
#include "TCPCommand.h"


#define VERSION "3.0.1 May 21, 2012   \r\n"


// *****************************************************************************
// Section: Configuration bits
// *****************************************************************************
	#pragma config POSCMOD  = XT            // Primary Oscillator using external crystal
	#pragma config FNOSC    = PRIPLL        // Oscillator Selection (primary osc or RC osc)
	#pragma config FPLLIDIV = DIV_2         // PLL Input Divider 
	#pragma config FPLLMUL  = MUL_20        // PLL Multiplier
	#pragma config FPLLODIV = DIV_1         // PLL Output Divider
	#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
	#pragma config FWDTEN   = OFF           // Watchdog Timer
	#pragma config WDTPS    = PS1           // Watchdog Timer Postscale
	#pragma config FCKSM    = CSECME        // Clock Switching & Fail Safe Clock Monitor
	#pragma config OSCIOFNC = OFF           // CLKO Enable

	#pragma config IESO     = OFF           // Internal/External Switch-over
	#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable
	#pragma config CP       = OFF           // Code Protect
	#pragma config BWP      = OFF           // Boot Flash Write Protect
	#pragma config PWP      = OFF           // Program Flash Write Protect
	#pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
	#pragma config DEBUG    = OFF           // Debugger Disabled for Starter Kit
	
	#pragma config FETHIO   = ON			// Ethernet IO Pin selection
	#pragma config FMIIEN	= OFF			// Ethernet MII Enable


__C32_UART = 0x1; 		//Redirect the STDIO for UART1 instead of the default of UART2


// Declare AppConfig structure and some other supporting stack variables
APP_CONFIG AppConfig;
static unsigned short wOriginalAppConfigChecksum;	// Checksum of the ROM defaults for AppConfig


// Prototypes
void InitAppConfig(void);
void InitializeBoard(void);
void UpdateRTC(void);
uint16 ADCRead(uint8 Channel);
float ReadCurrent(uint8 Channel);

//Define global variables
TIME Time;

uint16 Sec_counter;

boolean SecondFlag;
boolean MinuteFlag;
boolean HourFlag;
boolean HalfSecondFlag;
boolean PowerUpFlag;
boolean TickFlag;

float ChannelCurrentData[170];
float ChannelSquareSum;
float ChannelRMSCurrent[32];
uint8 i;
uint8 ChannelDataIndex;
uint8 ChannelIndex;

//
// Interrupt Service Routines
// 
void __ISR(_TIMER_2_VECTOR,ipl2) Timer2Handler(void)
{
	//Update the second counter with the 50 msec timer ticks
	Sec_counter++;
	TickFlag = TRUE;
	
	if(Sec_counter == 5010)
		HalfSecondFlag = TRUE;
	
	if(Sec_counter == 10020)
	{
		Sec_counter = 0;
		HalfSecondFlag = TRUE;
		UpdateRTC();
	}
	
	// Reset interrupt flag
	IFS0CLR = _IFS0_T2IF_MASK;
}


void _general_exception_handler(unsigned cause, unsigned status)
{
	Nop();
	Nop();
}


//
// Main application entry point.
//
void main(void)
{
static DWORD t = 0;
	//Set the local clock to initial time.  At the top of the first minute it will be reset
	//with a call to the time server.
	Time.Second = 55;
	Time.Minute = 11;
	Time.Hour = 14;
	Time.Month = 1;
	Time.Day = 1;
	Time.Year = 0;
	Sec_counter = 0;	
	SecondFlag = FALSE;
	MinuteFlag = FALSE;
	HourFlag = FALSE;
	HalfSecondFlag = FALSE;
	PowerUpFlag = TRUE;
	TickFlag = TRUE;
	ChannelDataIndex = 0;
	ChannelIndex = 32;
	i = 0;
	
	// Initialize application specific hardware
	InitializeBoard();

	// Initialize the timer to generate 10 msec ISRs
   	TickInit();

	//Initialize the EEPROM for use as the file system
	MPFSInit();	

	// Initialize Stack and application related NV variables into AppConfig.
	InitAppConfig();

	//Bring up menu to allow board configuration to be changed.  NOT USED
    //DoUARTConfig();

	// Initialize core stack layers (MAC, ARP, TCP, UDP) and
	// application modules (HTTP, SNMP, etc.)
    StackInit();

	// Initialize the TCP command interface for the WebServer2
	TCPCmdInit();
	
	//Display current board stack configuration info
	printf("\r\r\r");
	printf(VERSION);
	printf("Board configuration\r");
	printf("MAC is %x:%x:%x:%x:%x:%x\r", AppConfig.MyMACAddr.v[0], AppConfig.MyMACAddr.v[1], 
		AppConfig.MyMACAddr.v[2], AppConfig.MyMACAddr.v[3], AppConfig.MyMACAddr.v[4], 
		AppConfig.MyMACAddr.v[5]);
	printf("IP Address is %u.%u.%u.%u\r", AppConfig.MyIPAddr.v[0], AppConfig.MyIPAddr.v[1], 
		AppConfig.MyIPAddr.v[2], AppConfig.MyIPAddr.v[3]);
	printf("Gateway Address is %u.%u.%u.%u\r", AppConfig.MyGateway.v[0], 
		AppConfig.MyGateway.v[1], AppConfig.MyGateway.v[2], AppConfig.MyGateway.v[3]);
	printf("Mask is %u.%u.%u.%u\r", AppConfig.MyMask.v[0], AppConfig.MyMask.v[1], 
		AppConfig.MyMask.v[2], AppConfig.MyMask.v[3]);
	printf("DNS is %u.%u.%u.%u\r\r\r", AppConfig.PrimaryDNSServer.v[0], 
		AppConfig.PrimaryDNSServer.v[1], AppConfig.PrimaryDNSServer.v[2], 
		AppConfig.PrimaryDNSServer.v[3]);


    while(1)
    {
	   //The TickFlag is set every time the 99.8 usec interrupt from timer2 fires 
	    if(TickFlag == TRUE)
	    {
		    //The ChannelIndex variable is set to 0 at the top of each second
		    if(ChannelIndex < 32)
		    {
			    //Sample the given channel current measurements for one cycle of the 
			    //60 Hz cycle (167 * 99.8 usec is one cycle)
			    ChannelCurrentData[ChannelDataIndex++] = ReadCurrent(ChannelIndex);
			    
			    if(ChannelDataIndex >= 167)
			    {
				   	//We have a full cycle of data in memory so calculate the RMS value
				   	//Tests show this calculation takes about 330 usec
				    ChannelSquareSum = 0;
				    
				    //First calculate the sum of the square
				    for(i=0;i<167;i++)
					    ChannelSquareSum += pow(ChannelCurrentData[i],2);

					//Now calculate the RMS current
					ChannelRMSCurrent[ChannelIndex] = sqrt(ChannelSquareSum/167);
				    
				    ChannelIndex++;			//Advance to the next channel
				    ChannelDataIndex = 0;			    
				}		    
			} 
		    
		    TickFlag = FALSE;
		} 
	    
	    
	    
		//Do the one second tasks
		if((SecondFlag == TRUE))
		{				
						
			//Since it is the top of the second we start a data collection
			ChannelIndex = 0;
			ChannelDataIndex = 0;
			
			printf("%02u:%02u:%02u\r",Time.Hour,Time.Minute,Time.Second);		
			SecondFlag = FALSE;
			
			//For debugging send the current measurements to the serial port
			for(i=0;i<32;i++)
			{			
				printf("%6.3f  ",ChannelRMSCurrent[i]);
				if(i == 8) printf("\r");
			    if(i == 17) printf("\r");
			    if(i == 26) printf("\r");
			    if(i == 31) printf("\r");			
			}
			printf("\r");

					
		}		//if((SecondFlag == TRUE))



		//Do the one minute tasks
		if(MinuteFlag == TRUE)
		{				
			//Set the clock at the top of the first minute after powerup.  Allows the connection
			//to get stable and ready for traffic
			if(PowerUpFlag == TRUE)
			{
				//Set the clock based on the time determined by SNTP
				BinaryToDate(SNTPGetUTCSeconds(),&Time);
				printf("Time set to:  %02u/%02u/%02u %02u:%02u:%02u\r",Time.Month,Time.Day,Time.Year,Time.Hour,Time.Minute,Time.Second);
				PowerUpFlag = FALSE;
			}	
			
			MinuteFlag = FALSE;
		}		//if(MinuteFlag == TRUE)


		//Do the one hour tasks
		if(HourFlag == TRUE)
		{
			//Set the clock based on the time determined by SNTP
			BinaryToDate(SNTPGetUTCSeconds(),&Time);
				printf("Time set to:  %02u/%02u/%02u %02u:%02u:%02u\r",Time.Month,Time.Day,Time.Year,Time.Hour,Time.Minute,Time.Second);
			
			HourFlag = FALSE;
		}		//if(HourFlag == TRUE)
		
		
		//Flash the one second LED every second.  Also flash the user LED on the Max32 board
		if(HalfSecondFlag == TRUE)
		{
			mPORTAToggleBits(BIT_3);
			ONE_SEC_LED ^= 1;
			HalfSecondFlag = FALSE;
		}


        // This task performs normal stack task including checking
        // for incoming packet, type of packet and calling
        // appropriate stack entity to process it.
        StackTask();

        // This tasks invokes each of the core stack application tasks
        StackApplications();

		//This task processes the custom TCP/IP command interface for the home automation system
		TCPCmdProcess();

	}		//while(1)

}		//void main(void)






/****************************************************************************
  Function:
    static void InitializeBoard(void)

  Description:
    This routine initializes the hardware.  It is a generic initialization
    routine for many of the Microchip development boards, using definitions
    in HardwareProfile.h to determine specific initialization.

  Precondition:
    None

  Parameters:
    None - None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/
void InitializeBoard(void)
{

	//Configure the serial port	for the stdio
    UARTConfigure(UART1, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UART1, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, GetPeripheralClock(), 57600);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Configure the device for maximum performance but do not change the PBDIV
	// Given the options, this function will change the flash wait states, RAM
	// wait state and enable prefetch cache but will not change the PBDIV.
	// The PBDIV value is already set via the pragma FPBDIV option above..
	SYSTEMConfig(GetSystemClock(), SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

	//Disable the JTAG port since we are not using it
	mJTAGPortEnable(DEBUG_JTAGPORT_OFF);
	

	//Configure the LED bits as output and set to low
	ONE_SEC_LED_TRIS = 0;
	ONE_SEC_LED = 0;
	
	//Configure the analog mux
	MUX_SELECT_S0_TRIS;
	MUX_SELECT_S1_TRIS;
	MUX_SELECT_S2_TRIS;
	MUX_SELECT_0;	

	//The MAX32 board has an LED on A3 that we will toggle at a 1 sec rate
	mPORTAClearBits(BIT_3);
	mPORTASetPinsDigitalOut(BIT_3);
	mPORTAClearBits(BIT_3);
		
	
	//Configure timer2 for realtime keeping but setting it for a 99.8 usec interrupt rate
	T2CON = 0x0040;			//Timer off, 16 bit mode, /16 from pperipheral clk
	TMR2 = 0;
	PR2 = TIMER2_MATCH;		//Configure for the 99.8 usec rate 
	IFS0CLR = _IFS0_T2IF_MASK;
	IEC0SET = _IEC0_T2IE_MASK;
	
	//Configure Timer2 Interrupt
	INTEnable(INT_T2 , INT_ENABLED);
	INTSetVectorPriority(INT_TIMER_2_VECTOR, INT_PRIORITY_LEVEL_2);
	INTSetVectorSubPriority(INT_TIMER_2_VECTOR, INT_SUB_PRIORITY_LEVEL_0);

	//Configure for multi-vectored mode
	INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);

	//Enable interrupts
	INTEnableInterrupts();		
		
	//Timer is ready so turn it on
	T2CON = 0x8040;			//Timer on, 16 bit mode, /16 from pperipheral clk

	
	//Configure the ADC
	AD1CON1bits.ON = 1;			//Turn ADC on
	AD1CON1bits.SIDL = 0;		//Continue in idle mode
	AD1CON1bits.FORM = 0;		//Integer format
	AD1CON1bits.SSRC = 7;		//Auto Convert mode
	AD1CON1bits.CLRASAM = 0;	//Normal buffer operation
	AD1CON1bits.ASAM = 0;		//Sampling starts when SAMP bit is set
	
	AD1CON2bits.VCFG = 0;		//Vr+ = Vdd, Vr- = Vss
	AD1CON2bits.OFFCAL = 0;		//Disable offset calibration
	AD1CON2bits.CSCNA = 0;		//Don't scan inputs
	AD1CON2bits.BUFS = 0;		//Buffer fill status
	AD1CON2bits.SMPI = 0;		//Interrupt at completion of each sample/convert sequence
	AD1CON2bits.BUFM = 0;		//16 bit word
	AD1CON2bits.ALTS = 0;		//Use Mux A only 
	
	AD1CON3bits.ADRC = 0;		//Use PBclk 
	AD1CON3bits.SAMC = 3;		//Use 3 Tad
	AD1CON3bits.ADCS = 7;		//Use 16 prescale (200 nsec Tad)
	
	AD1CHSbits.CH0NB = 0;		//Negative input is Vr- (don't care)
	AD1CHSbits.CH0SB = 0;		//Mux B select (don't care)
	AD1CHSbits.CH0NA = 0;		//Negative input is Vr-
	AD1CHSbits.CH0SA = 4;		//Initial channel selection
	
	AD1PCFG = 0x3f;				//Lower 5 channels used for analog inputs
	
	AD1CSSLbits.CSSL = 0;		//Scan select (don't care)

}

/*********************************************************************
 * Function:        void InitAppConfig(void)
 *
 * PreCondition:    MPFSInit() is already called.
 *
 * Input:           None
 *
 * Output:          Write/Read non-volatile config variables.
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/

//Read the IP configuration from EEPROM.  If it is different from the default
//values hard coded the EEPROM is updated.

static ROM BYTE SerializedMACAddress[6] = {MY_DEFAULT_MAC_BYTE1, MY_DEFAULT_MAC_BYTE2, MY_DEFAULT_MAC_BYTE3, MY_DEFAULT_MAC_BYTE4, MY_DEFAULT_MAC_BYTE5, MY_DEFAULT_MAC_BYTE6};

void InitAppConfig(void)
{
//unsigned char vNeedToSaveDefaults = 0;
	NVM_VALIDATION_STRUCT NVMValidationStruct;

	while(1)
	{
		// Start out zeroing all AppConfig bytes to ensure all fields are 
		// deterministic for checksum generation
		memset((void*)&AppConfig, 0x00, sizeof(AppConfig));
		
		AppConfig.Flags.bIsDHCPEnabled = TRUE;
		AppConfig.Flags.bInConfigMode = TRUE;
		memcpypgm2ram((void*)&AppConfig.MyMACAddr, (ROM void*)SerializedMACAddress, sizeof(AppConfig.MyMACAddr));

		AppConfig.MyIPAddr.Val = MY_DEFAULT_IP_ADDR_BYTE1 | MY_DEFAULT_IP_ADDR_BYTE2<<8ul | MY_DEFAULT_IP_ADDR_BYTE3<<16ul | MY_DEFAULT_IP_ADDR_BYTE4<<24ul;
		AppConfig.DefaultIPAddr.Val = AppConfig.MyIPAddr.Val;
		AppConfig.MyMask.Val = MY_DEFAULT_MASK_BYTE1 | MY_DEFAULT_MASK_BYTE2<<8ul | MY_DEFAULT_MASK_BYTE3<<16ul | MY_DEFAULT_MASK_BYTE4<<24ul;
		AppConfig.DefaultMask.Val = AppConfig.MyMask.Val;
		AppConfig.MyGateway.Val = MY_DEFAULT_GATE_BYTE1 | MY_DEFAULT_GATE_BYTE2<<8ul | MY_DEFAULT_GATE_BYTE3<<16ul | MY_DEFAULT_GATE_BYTE4<<24ul;
		AppConfig.PrimaryDNSServer.Val = MY_DEFAULT_PRIMARY_DNS_BYTE1 | MY_DEFAULT_PRIMARY_DNS_BYTE2<<8ul  | MY_DEFAULT_PRIMARY_DNS_BYTE3<<16ul  | MY_DEFAULT_PRIMARY_DNS_BYTE4<<24ul;
		AppConfig.SecondaryDNSServer.Val = MY_DEFAULT_SECONDARY_DNS_BYTE1 | MY_DEFAULT_SECONDARY_DNS_BYTE2<<8ul  | MY_DEFAULT_SECONDARY_DNS_BYTE3<<16ul  | MY_DEFAULT_SECONDARY_DNS_BYTE4<<24ul;
	
	
		// Load the default NetBIOS Host Name
		memcpypgm2ram(AppConfig.NetBIOSName, (ROM void*)MY_DEFAULT_HOST_NAME, 16);
		FormatNetBIOSName(AppConfig.NetBIOSName);
	

		// Compute the checksum of the AppConfig defaults as loaded from ROM
		wOriginalAppConfigChecksum = CalcIPChecksum((BYTE*)&AppConfig, sizeof(AppConfig));

/*		We are not using a EEPROM in this design so the following is not requried.

		NVM_VALIDATION_STRUCT NVMValidationStruct;

		// Check to see if we have a flag set indicating that we need to 
		// save the ROM default AppConfig values.
		if(vNeedToSaveDefaults)
			SaveAppConfig(&AppConfig);
	
		// Read the NVMValidation record and AppConfig struct out of EEPROM/Flash
		printf("Reading IP configuration from EEPROM. \r");
		XEEReadArray(0x0000, (BYTE*)&NVMValidationStruct, sizeof(NVMValidationStruct));
		XEEReadArray(sizeof(NVMValidationStruct), (BYTE*)&AppConfig, sizeof(AppConfig));

		// Check EEPROM/Flash validitity.  If it isn't valid, set a flag so 
		// that we will save the ROM default values on the next loop 
		// iteration.
		if((NVMValidationStruct.wConfigurationLength != sizeof(AppConfig)) ||
			//The following line checks for a change in the default vs EEPROM data
		   (NVMValidationStruct.wOriginalChecksum != wOriginalAppConfigChecksum) ||
		   //The following line checks the EEPROM calculated CRC vs the stored CRC
		   (NVMValidationStruct.wCurrentChecksum != CalcIPChecksum((BYTE*)&AppConfig, sizeof(AppConfig))))
		{
			// Check to ensure that the vNeedToSaveDefaults flag is zero, 
			// indicating that this is the first iteration through the do 
			// loop.  If we have already saved the defaults once and the 
			// EEPROM/Flash still doesn't pass the validity check, then it 
			// means we aren't successfully reading or writing to the 
			// EEPROM/Flash.  This means you have a hardware error and/or 
			// SPI configuration error.

			printf("EEPROM does not equal default or has an error...\r");
			
			if(vNeedToSaveDefaults)
			{
				printf("Error writing IP configuration data to EEPROM... stopping\r");

				while(1);
			}
			
			// Set flag and restart loop to load ROM defaults and save them
			vNeedToSaveDefaults = 1;
			continue;
		}
		
		// If we get down here, it means the EEPROM/Flash has valid contents 
		// and either matches the ROM defaults or previously matched and 
		// was run-time reconfigured by the user.  In this case, we shall 
		// use the contents loaded from EEPROM/Flash.
*/
		
		break;

	}	//while(1)
}

/*		We are not using a EEPROM in this design so the following is not requried.

void SaveAppConfig(const APP_CONFIG *ptrAppConfig)
{
	NVM_VALIDATION_STRUCT NVMValidationStruct;

	printf("EEPROM write started. \r");

	// Ensure adequate space has been reserved in non-volatile storage to 
	// store the entire AppConfig structure.  If you get stuck in this while(1) 
	// trap, it means you have a design time misconfiguration in TCPIPConfig.h.
	// You must increase MPFS_RESERVE_BLOCK to allocate more space.
	#if defined(STACK_USE_MPFS2)
		if(sizeof(NVMValidationStruct) + sizeof(AppConfig) > MPFS_RESERVE_BLOCK)
			while(1);
	#endif

	// Get proper values for the validation structure indicating that we can use 
	// these EEPROM/Flash contents on future boot ups
	NVMValidationStruct.wOriginalChecksum = wOriginalAppConfigChecksum;
	NVMValidationStruct.wCurrentChecksum = CalcIPChecksum((BYTE*)ptrAppConfig, sizeof(APP_CONFIG));
	NVMValidationStruct.wConfigurationLength = sizeof(APP_CONFIG);

	// Write the validation struct and current AppConfig contents to EEPROM
	XEEBeginWrite(0x0000);
	XEEWriteArray((BYTE*)&NVMValidationStruct, sizeof(NVMValidationStruct));
	XEEWriteArray((BYTE*)ptrAppConfig, sizeof(APP_CONFIG));

	printf("EEPROM write complete. \r");

}
*/


/**

	Update the real time clock structure and set time event flags.  The
	event flags are cleared with the main loop task completes

	@return void

*/
void UpdateRTC(void)
{

	SecondFlag = TRUE;

	Time.Second++;
	if(Time.Second >= 60)
	{
		MinuteFlag = TRUE;
		Time.Second = 0;
		Time.Minute++;
		if(Time.Minute >= 60)
		{
			HourFlag = TRUE;
			Time.Minute = 0;
			Time.Hour++;
			if(Time.Hour >= 24)
			{
				Time.Hour = 0;
			}
		}
	}

}

/**
	For the current project we are not using an EEPROM for web page storage so the following is 
	really not needed but is left here for future implementations.  For the current project
	the webpage is complied as a c file and loaded into flash and can not be changed via FTP

	Webpages will be uploaded using FTP.  The FTP server will call this routine 
	to verify the id of the user.  We check the user name and password and return
	the results

	@return boolean - TRUE if user/password are vaild, FASLE otherwise

*/

ROM char FTP_USER_NAME[]    = "admin";
ROM char FTP_USER_PASS[]    = "microchip";
#undef FTP_USER_NAME_LEN
#define FTP_USER_NAME_LEN   (sizeof(FTP_USER_NAME)-1)
#define FTP_USER_PASS_LEN   (sizeof(FTP_USER_PASS)-1)

BOOL FTPVerify(char *login, char *password)
{

printf("login = %s, password=%s\r",login,password);

    if ( !memcmppgm2ram(login, (ROM void*)FTP_USER_NAME, FTP_USER_NAME_LEN) )
    {
        if ( !memcmppgm2ram(password, (ROM void*)FTP_USER_PASS, FTP_USER_PASS_LEN) )
            return TRUE;
    }
    return FALSE;
}




/**
	This routine reads the selected ADC channel 

	@param uint8 Channel - ADC channel to read
  	@return uint16 ADC reading
*/
uint16 ADCRead(uint8 Channel)
{
	
	AD1CHSbits.CH0SA = Channel;				//Select the desired channel
	
	AD1CON1bits.SAMP = 1;					//Start the conversion
	
	while(AD1CON1bits.DONE == 0){}			//Wait for the conversion to complete
	
	return ADC1BUF0;
}	



/**
	This routine reads the current from the selected channel 

	@param uint8 Channel - Current measurement channel to read (0-31)
  	@return uint16 Current reading
*/
float ReadCurrent(uint8 Channel)
{
	int16 ADCReading;
	float ChannelCurrent;
	
	switch(Channel)
	{
		case 0:
		
			MUX_SELECT_5;
			ADCReading = (int16)ADCRead(0) - CHAN0_CAL;
			ChannelCurrent = (float)ADCReading * CHAN0_GAIN;
			break;
			
		case 1:
		
			MUX_SELECT_3;	
			ADCReading = (int16)ADCRead(0)- CHAN1_CAL;
			ChannelCurrent = (float)ADCReading * CHAN1_GAIN;
			break;				
			
		case 2:

			MUX_SELECT_2;		
			ADCReading = (int16)ADCRead(2)- CHAN2_CAL;
			ChannelCurrent = (float)ADCReading * CHAN2_GAIN;
			break;				
			
		case 3:

			MUX_SELECT_4;		
			ADCReading = (int16)ADCRead(1)- CHAN3_CAL;
			ChannelCurrent = (float)ADCReading * CHAN3_GAIN;
			break;				
			
		case 4:

			MUX_SELECT_0;		
			ADCReading = (int16)ADCRead(1)- CHAN4_CAL;
			ChannelCurrent = (float)ADCReading * CHAN4_GAIN;;
			break;				
			
		case 5:
	
			MUX_SELECT_1;	
			ADCReading = (int16)ADCRead(1)- CHAN5_CAL;
			ChannelCurrent = (float)ADCReading * CHAN5_GAIN;
			break;				
			
		case 6:

			MUX_SELECT_6;		
			ADCReading = (int16)ADCRead(2)- CHAN6_CAL;
			ChannelCurrent = (float)ADCReading * CHAN6_GAIN;
			break;				
			
		case 7:

			MUX_SELECT_7;		
			ADCReading = (int16)ADCRead(2)- CHAN7_CAL;
			ChannelCurrent = (float)ADCReading * CHAN7_GAIN;
			break;				
			
		case 8:

			MUX_SELECT_0;		
			ADCReading = (int16)ADCRead(2)- CHAN8_CAL;
			ChannelCurrent = (float)ADCReading * CHAN8_GAIN;
			break;				
			
		case 9:
	
			MUX_SELECT_4;	
			ADCReading = (int16)ADCRead(3)- CHAN9_CAL;
			ChannelCurrent = (float)ADCReading * CHAN9_GAIN;
			break;				
			
		case 10:

			MUX_SELECT_5;		
			ADCReading = (int16)ADCRead(3)- CHAN10_CAL;
			ChannelCurrent = (float)ADCReading * CHAN10_GAIN;
			break;				
			
		case 11:

			MUX_SELECT_3;		
			ADCReading = (int16)ADCRead(3)- CHAN11_CAL;
			ChannelCurrent = (float)ADCReading * CHAN11_GAIN;
			break;				
			
		case 12:

			MUX_SELECT_6;		
			ADCReading = (int16)ADCRead(0)- CHAN12_CAL;
			ChannelCurrent = (float)ADCReading * CHAN12_GAIN;
			break;				
			
		case 13:

			MUX_SELECT_7;		
			ADCReading = (int16)ADCRead(0)- CHAN13_CAL;
			ChannelCurrent = (float)ADCReading * CHAN13_GAIN;
			break;				
			
		case 14:
	
			MUX_SELECT_2;	
			ADCReading = (int16)ADCRead(0)- CHAN14_CAL;
			ChannelCurrent = (float)ADCReading * CHAN14_GAIN;
			break;				
			
		case 15:

			MUX_SELECT_1;
			ADCReading = (int16)ADCRead(2)- CHAN15_CAL;
			ChannelCurrent = (float)ADCReading * CHAN15_GAIN;
			break;				
			
		case 16:

			MUX_SELECT_5;
			ADCReading = (int16)ADCRead(1)- CHAN16_CAL;
			ChannelCurrent = (float)ADCReading * CHAN16_GAIN;
			break;				
			
		case 17:

			MUX_SELECT_3;
			ADCReading = (int16)ADCRead(1)- CHAN17_CAL;
			ChannelCurrent = (float)ADCReading * CHAN17_GAIN;
			break;				
			
		case 18:
	
			MUX_SELECT_2;
			ADCReading = (int16)ADCRead(3)- CHAN18_CAL;
			ChannelCurrent = (float)ADCReading * CHAN18_GAIN;
			break;				
			
		case 19:

			MUX_SELECT_4;
			ADCReading = (int16)ADCRead(2)- CHAN19_CAL;
			ChannelCurrent = (float)ADCReading * CHAN19_GAIN;
			break;				
			
		case 20:

			MUX_SELECT_5;
			ADCReading = (int16)ADCRead(2)- CHAN20_CAL;
			ChannelCurrent = (float)ADCReading * CHAN20_GAIN;
			break;				
			
		case 21:

			MUX_SELECT_3;
			ADCReading = (int16)ADCRead(2)- CHAN21_CAL;
			ChannelCurrent = (float)ADCReading * CHAN21_GAIN;
			break;				
			
		case 22:
	
			MUX_SELECT_6;
			ADCReading = (int16)ADCRead(3)- CHAN22_CAL;
			ChannelCurrent = (float)ADCReading * CHAN22_GAIN;
			break;				
			
		case 23:

			MUX_SELECT_7;
			ADCReading = (int16)ADCRead(3)- CHAN23_CAL;
			ChannelCurrent = (float)ADCReading * CHAN23_GAIN;
			break;				
			
		case 24:

			MUX_SELECT_2;
			ADCReading = (int16)ADCRead(1)- CHAN24_CAL;
			ChannelCurrent = (float)ADCReading * CHAN24_GAIN;
			break;				
			
		case 25:
	
			MUX_SELECT_4;
			ADCReading = (int16)ADCRead(0)- CHAN25_CAL;
			ChannelCurrent = (float)ADCReading * CHAN25_GAIN;
			break;				
			
		case 26:

			MUX_SELECT_0;
			ADCReading = (int16)ADCRead(0)- CHAN26_CAL;
			ChannelCurrent = (float)ADCReading * CHAN26_GAIN;
			break;				
			
		case 27:

			MUX_SELECT_1;
			ADCReading = (int16)ADCRead(0)- CHAN27_CAL;
			ChannelCurrent = (float)ADCReading * CHAN27_GAIN;
			break;				
			
		case 28:

			MUX_SELECT_6;
			ADCReading = (int16)ADCRead(1)- CHAN28_CAL;
			ChannelCurrent = (float)ADCReading * CHAN28_GAIN;
			break;				
			
		case 29:
	
			MUX_SELECT_7;
			ADCReading = (int16)ADCRead(1)- CHAN29_CAL;
			ChannelCurrent = (float)ADCReading * CHAN29_GAIN;
			break;				
			
		case 30:
	
			MUX_SELECT_0;
			ADCReading = (int16)ADCRead(3)- CHAN30_CAL;
			ChannelCurrent = (float)ADCReading * CHAN30_GAIN;
			break;				
		
		case 31:

			MUX_SELECT_1;
			ADCReading = (int16)ADCRead(3)- CHAN31_CAL;
			ChannelCurrent = (float)ADCReading * CHAN31_GAIN;
			break;
			
		default:

			MUX_SELECT_0;
			ChannelCurrent = 0;
			break;					
	}	
	
	return ChannelCurrent;
}	



/**
	This routine is the call back function from TCPProcess.  It decodes the received 
	commands and returns true if the command was a valid command.  Actions required 
	by the command are taken and return data is provided.


  	@return boolean - TRUE if command was valid, FALSE otherwise

*/

boolean TCPCommandDecode(uint8 *CmdString,uint8 *RespString)
{

	uint8 NumberString[10];
	uint8 Index;

	//Process the status message
	//We split the status command in to two halves since we can only send a 100 byte message at
	//a time.
	if(strcmppgm2ram(CmdString,"EMS_statusA") == 0)
	{
		//We have a "EMS_statusA" command
		printf("%02u:%02u:%02u  EMS status A request received.\r",Time.Hour,Time.Minute,Time.Second);

		sprintf(RespString, "EMS_statusA_");		
		
		for(Index=0;Index<16;Index++)
		{
			NumberString[0] = 0;
			if(Index != 15)
				sprintf(NumberString,"%3.1f,",ChannelRMSCurrent[Index]);
			else
				sprintf(NumberString,"%3.1f",ChannelRMSCurrent[Index]);
				
			strcat(RespString,NumberString);
		}
			
		return TRUE;
	}
	
	if(strcmppgm2ram(CmdString,"EMS_statusB") == 0)
	{
		//We have a "EMS_statusB" command
		printf("%02u:%02u:%02u  EMS status B request received.\r",Time.Hour,Time.Minute,Time.Second);

		sprintf(RespString, "EMS_statusB_");		
		
		for(Index=16;Index<32;Index++)
		{
			NumberString[0] = 0;
			if(Index != 31)
				sprintf(NumberString,"%3.1f,",ChannelRMSCurrent[Index]);
			else
				sprintf(NumberString,"%3.1f",ChannelRMSCurrent[Index]);
				
			strcat(RespString,NumberString);
		}
			
		return TRUE;
	}	

	//Indicate an error for any other message besides the status message
	return FALSE;
}



