/*////////////////////////////////////////////////////////////////////////////
	PROJECT:	PIC Ethernet
	COPYRIGHT	 2006 Dean J. Boman

	FILE:		TCPCommand.c
	AUTHOR:		Dean J. Boman

	OVERVIEW:
		This implements the TCP command interface for the PIC Ethernet board. A 
		port is opened and monitored for an external connection.  When a connection
		is detected the input data is examined for a valid command.  If a command is 
		found the command is executed and any return data sent back to the source.

	REVISION HISTORY:

	Oct. 2, 2006	Initial development
	May 16, 2012	Update for new stack


*/////////////////////////////////////////////////////////////////////////////

#include "TCPIP Stack/TCPIP.h"
#include "typedef.h"
#include "TCPCommand.h"

//Command Error string (null terminated)
ROM uint8 CMD_ERROR[] = "Error\0";

// TCPCmd states 
typedef enum _SM_TCPCmd
{
    SM_TCPCmd_IDLE,
    SM_TCPCmd_SEND,
    SM_TCPCmd_DISCONNECT
} SM_TCPCmd;


// TCPCmd Connection Info 
typedef struct _TCPCmd_INFO
{
    TCP_SOCKET socket;
    SM_TCPCmd state;
	boolean CommandValid;
} TCPCmd_INFO;

static TCPCmd_INFO TCPCmd;




/**
	This routine intializes the TCP Command interface by opening a command port
	for the connection.  This routine must be called during system startup.


  	@return void

*/
void TCPCmdInit(void)
{
	TCPCmd.socket = TCPListen(TCPCmd_PORT);
	TCPCmd.state = SM_TCPCmd_IDLE;
}




/**
	This routine checks for a connection on the listening port.  If a connection is
	present the input data is extracted from the socket and passed to the command processor.
	Any return data or message is sent back to the connection.  This routine must be called
	perodically by the main application loop.


  	@return void

*/

void TCPCmdProcess(void)
{
	boolean Continue;
	uint8 TCPCmdData[TCP_CMD_DATA_LEN + 1];
	uint8 TCPCmdLength;
	uint8 TCPRespData[TCP_RESP_DATA_LEN + 1];
	uint8 TCPRespLength;
	uint8 OutputChar;
	uint8 *OutputStringPointer;

	do
	{
		Continue = FALSE;

		//Check to see if the connection has been broken.  If so return
		//to the idle state
		if(TCPIsConnected(TCPCmd.socket) == FALSE)
		{
			TCPCmd.state = SM_TCPCmd_IDLE;
			break;
		}

		//If we get beyond this point we have received a connection on the command port

		switch(TCPCmd.state)
		{

		case SM_TCPCmd_IDLE:
		
			//Check to see if input data is available
			if(TCPIsGetReady(TCPCmd.socket))
			{
				Continue = TRUE;

				//Move the data from the socket to the TCPCmdData variable
                TCPCmdLength = 0;
				while(TCPCmdLength < TCP_CMD_DATA_LEN)
				{
					//Move data as long as data is available
					if(TCPGet(TCPCmd.socket,&TCPCmdData[TCPCmdLength++]) == FALSE)
					{
						TCPCmdLength--;	//Last byte is not valid so backup one location
						break;
					}
				} 

                TCPCmdData[TCPCmdLength] = '\0';	//Null terminate the string

				//Dump any data that may be remaining in the socket.  There should 
				//never be any since we read the socket until it was empty
                TCPDiscard(TCPCmd.socket);		


				//This is a call back function to the main routine to precess the 
				//input data
				TCPCmd.CommandValid = TCPCommandDecode(&TCPCmdData[0],&TCPRespData[0]);
				
				TCPCmd.state = SM_TCPCmd_SEND;
			}

			break;



		case SM_TCPCmd_SEND:

			//We are now ready to send a message back on the socket.

			//If command was not valid copy error messge to out bound data array
			if(TCPCmd.CommandValid == FALSE)
				strcpypgm2ram(TCPRespData,CMD_ERROR);

			OutputStringPointer = &TCPRespData[0];


			if(TCPIsPutReady(TCPCmd.socket))
			{
				//Send the data to the socket until a null byte is found or we have sent
				//the entire buffer.  We don't check for a full socket since the return 
				//messages are very small.

				TCPRespLength = 0;
				while((OutputChar = *OutputStringPointer++) && 
					(TCPRespLength < TCP_RESP_DATA_LEN))
				{
					TCPPut(TCPCmd.socket,OutputChar);
					TCPRespLength++;
				}

				TCPFlush(TCPCmd.socket);
			}

			TCPCmd.state = SM_TCPCmd_DISCONNECT;

			break;


		case SM_TCPCmd_DISCONNECT:

            if(TCPIsConnected(TCPCmd.socket))
            {
				//Wait until the socket is ready for more data before disconnecting
				//since the socket will not show ready until the message is acknowledeged
				//by the remote connection.
                if(TCPIsPutReady(TCPCmd.socket))
                    TCPDisconnect(TCPCmd.socket);

            }

			TCPCmd.state = SM_TCPCmd_IDLE;

            break;

		}

	}while(Continue == TRUE);

}





